% Initialization for the Launch Vehicle 6-DOF Simulation 
clear all
d2r=pi/180; r2d=180/pi;
%unit conversions
m2ft=3.280840;
kg2slug = 0.068522;
kgpm3_to_slugpft3 = kg2slug / m2ft^3;
pa2psi = 1/6894.757000; K2R = 1.8;
lbf2N= 4.44822; ft2m= 0.3048;
decim=250;                                             % Plots Resolution
lat0=34.57; long0=-120.633; h0=0;                      % Initial Latit, Long, Altitude
In_Posit=[lat0,long0,h0]; x0=0; a0=0*d2r;                        
gm0=89.98*d2r; V0=0.2;                                 % Init Gamma, speed
Euler_0=[90, 89.98, 0]*d2r;                            % Init Euler Attitude 
pqr_0=[0, 0, 0];                                       % Init Rates [0, 0.0,0] (rad/sec)
Uvw_0=[1,0,0]*V0;                                      % X,Y,Z Veloc Comp Body (ft/sec)
x_cg= 59.329; y_cg=0.0;  z_cg=0.0;                     % Initial CG location
cg=[x_cg, y_cg, z_cg];
x_ref=120.142; y_ref=0; z_ref=0;                       % Aero Moments Reference
mrc=[x_ref, y_ref, z_ref];
Sref=44.4146; ch=7.52; sp=7.52;                        % Reference Area, Length
accel=[97.3083,0,0];                                   % Accelerometer Location
Linact=0;                                              % 1-Linear, 0-NonLinear Actuator Switch
Iflex= 1;                                              % 1-Flexibility, 0-No Flexing Switch
Islosh=1;                                              % 1-With Slosh,  0-No Slosh Switch
TqLoad=1;                                              % 1-TorqueLoad,  0-No Torque Load
thpnd=15; velpnd=3;                                    % Init Pendulum Angle (deg), Veloc (ft/s)
Cdelay=0.005;                                          % Computational Delay (sec)
Mdelay=0.005;                                          % Measurements  Delay (sec)
AcNoise=0.1;                                           % Actuator Mesurement Noise 0.1 (deg)
Nmch=16; Nalf=9; Nbet=7;

% Events Timing
T1a= 168.0;                                            % Just Before MECO
T1b= 168.5;                                            % Just After  MECO
T2a= 171.2;                                            % Just Before Separation
T2b= 171.3;                                            % Just After  Separation
T3a= 176.5;                                            % Just Before Stage-2 Ignition
T3b= 176.6;                                            % Just After  Stage-2 Ignition
pre_sep_wait = 4;
post_sep_wait= 4;
stage_to_fairing_sep_time = 34.7;

% Early termination flags
s1_only = 0;

% Load the Commands from Trajectory ..........................
load Commands.mat -ascii
timin= Commands(:,1);                                  % Time In
thecm= Commands(:,2);                                  % Theta Command (deg)
alfcm= Commands(:,3);                                  % Alfa  Command (deg)
gamcm= Commands(:,4);                                  % Gamma Command (deg)
headc= Commands(:,5);                                  % Heading Command (deg)
thrs1= Commands(:,6);                                  % Engine 1 Thrust (lb)
thrs2= Commands(:,7);                                  % Engine 2 Thrust (lb)
mssin= Commands(:,8);                                  % Vehicle Mass
incln= Commands(:,9);                                  % Inclination
Eul1 = Commands(:,10);                                 % Euler(1)
Eul2 = Commands(:,11);                                 % Euler(2)
Eul3 = Commands(:,12);                                 % Euler(3)

% Load Guidance Commands ..........................
load Commands_2.mat -ascii
timi2= Commands_2(:,1);                                % Time-2
JJ1  = Commands_2(:,2);                                % Momentum -x Vector
JJ2  = Commands_2(:,3);                                % Momentum -y
JJ3  = Commands_2(:,4);                                % Momentum -z
AMm  = Commands_2(:,5);                                % Angul-Momentum Magn
Enrg = Commands_2(:,6);                                % Energy
dEdJ = Commands_2(:,7);                                % dE/d(AM)
KE   = Commands_2(:,8);                                % Kinetic Energy

iend = length(Commands_2(:,1));
qtc1 = interp1(timi2(1:iend),Commands_2(1:iend, 9),timi2,'linear','extrap'); % Quat1
qtc2 = interp1(timi2(1:iend),Commands_2(1:iend,10),timi2,'linear','extrap'); % Quat1
qtc3 = interp1(timi2(1:iend),Commands_2(1:iend,11),timi2,'linear','extrap'); % Quat1
qtc4 = interp1(timi2(1:iend),Commands_2(1:iend,12),timi2,'linear','extrap'); % Quat1

% Load the Mass Properties
load Mass_Props -ascii
msspr= Mass_Props(:,1);                                 % Vehi Mass from Mass Props
Ixxpr= Mass_Props(:,2);                                 % Ixx from Mass Props
Iyypr= Mass_Props(:,3);                                 % Iyy from Mass Props
Izzpr= Mass_Props(:,4);                                 % Izz from Mass Props
Ixypr= Mass_Props(:,5);                                 % Ixy from Mass Props
Ixzpr= Mass_Props(:,6);                                 % Ixz from Mass Props
Iyzpr= Mass_Props(:,7);                                 % Iyz from Mass Props
Xcgpr= Mass_Props(:,8);                                 % X_cg from Mass Props
Ycgpr= Mass_Props(:,9);                                 % Y_cg from Mass Props
Zcgpr= Mass_Props(:,10);                                % Z_cg from Mass Props

% Load the Slosh Data from File
load Slosh -ascii
Mvsls= Slosh(:,1);                                      % Vehi Mass (slug)
Mslox= Slosh(:,2);                                      % LOX  Mass (slug)
Msfue= Slosh(:,3);                                      % Fuel Mass (slug)
Ztlox= Slosh(:,6);                                      % LOX  Mass Damping
Ztfue= Slosh(:,7);                                      % Fuel Mass Damping
Xlox = Slosh(:,8);                                      % LOX  Mass Locat (ft)
Xfuel= Slosh(:,9);                                      % Fuel Mass Locat (ft)
Rlox = Slosh(:,10);                                     % LOX  Pendul Length (ft)
Rfuel= Slosh(:,11);                                     % Fuel Pendul Length (ft)

% Engine Data
me1=9.325; me2= 13.927;                                 % 1st and 2nd Stage Engine Mass (slg)
Ie1=28.17; Ie2= 258.74;                                 % 1st and 2nd Stage Engine Inertia (slg-ft2)
Le1=1.203; Le2= 2.892;                                  % 1st and 2nd Stage Engine CG to Gimbal Arm (ft)
Xgmb1=10.7062;                                          % X Gimal Location for Stage-1
Xgmb2=82.538;                                           % X Gimal Location for Stage-2

% Stage-1 Engine Gimbals ..............................................
tvc1= [Xgmb1,  2.4638,  -1.0202];                       % Eng# 1 TVC Location (Fixed)
tvc2= [Xgmb1,  1.0202,  -2.4638];                       % Eng# 2 TVC Location
tvc3= [Xgmb1, -1.0202,  -2.4638];                       % Eng# 3 TVC Location            
tvc4= [Xgmb1, -2.4638,  -1.0202];                              
tvc5= [Xgmb1, -2.4638,   1.0202];                              
tvc6= [Xgmb1, -1.0202,   2.4638];                               
tvc7= [Xgmb1,  1.0202,   2.4638];                           
tvc8= [Xgmb1,  2.4638,   1.0202];                          
tvc9= [Xgmb1,  0.0,      0.0];                          % Fixed Eng# 9 Location

% RCS Coasting and Stage-2 ............................................
LBrcs= [82, -3.375, 0.0];                               % Left RCS Location
RBrcs= [82, +3.375, 0.0];                               % Rght RCS Location
TBrcs= [82, 0.0, -3.375];                               % Top  RCS Location
BBrcs= [82, 0.0, +3.375];                               % Botm RCS Location
tvc10= [Xgmb2, 0.0,  0.0];                              % TVC Engine Gimbal
Isp=65;                                                 % Isp in (sec)

% RCS Jet Thrust Directions --------------------------------------
nt=8;                                                   % Number of Jets
Jdir= zeros(3,nt); Jloc=Jdir;
Jdir(:, 1)= [0.0,  +1.0,   0.0]';                       % Top  +Y
Jdir(:, 2)= [0.0,  +1.0,   0.0]';                       % Botm +Y
Jdir(:, 3)= [0.0,  -1.0,   0.0]';                       % Top  -Y
Jdir(:, 4)= [0.0,  -1.0,   0.0]';                       % Botm -Y

Jdir(:, 5)= [0.0,   0.0,  +1.0]';                       % Left  +Z
Jdir(:, 6)= [0.0,   0.0,  +1.0]';                       % Right +Z
Jdir(:, 7)= [0.0,   0.0,  -1.0]';                       % Left  -Z
Jdir(:, 8)= [0.0,   0.0,  -1.0]';                       % Right -Z

% RCS Jet Thrust Locations ---------------------------------------
Jloc(:, 1) = [83.06,  0.0, -3.75]';                     % Top  +Y
Jloc(:, 2) = [83.06,  0.0, +3.75]';                     % Botm +Y
Jloc(:, 3) = [83.06,  0.0, -3.75]';                     % Top  -Y
Jloc(:, 4) = [83.06,  0.0, +3.75]';                     % Botm -Y

Jloc(:, 5) = [83.06, -3.75,  0.0]';                     % Left  +Z
Jloc(:, 6) = [83.06, +3.75,  0.0]';                     % Right +Z
Jloc(:, 7) = [83.06, -3.75,  0.0]';                     % Left  -Z
Jloc(:, 8) = [83.06, +3.75,  0.0]';                     % Right -Z

% Mixing Logic TVC Matrix for Stage-1 .................................
Kmix1=[-1,       -1,   0; ...
       -0.4141,  -1,   0; ...
        0.4141,  -1,   0; ...
        1,       -1,   0; ...
        1,       -1,   0; ...
        0.4141,  -1,   0; ...
       -0.4141,  -1,   0; ...
       -1,       -1,   0; ...
        0.4141,   0,  -1; ...
        1,        0,  -1; ...
        1,        0,  -1; ...
        0.4141,   0,  -1; ...
       -0.4141,   0,  -1; ...
       -1,        0,  -1; ...
       -1,        0,  -1; ...
       -0.4141,   0,  -1];
    
% Mixing Logic TVC Matrix for Stage-2
Kmix2=[ 0,   -1,    0; ...
        0,    0,   -1];

% Load Aero Data for the Base Vehicle
[CX, CY, CZ, Cl, Cm, Cn] = Stage1_Aero_Coeff;                                
Mach1=[0.0,0.3,0.6,0.8,0.85,0.9,0.95,1.0,1.05,1.1,1.2,1.45,2.0,3.5,5.0,30];
Beta1=[-8, -4, -2, 0, +2, +4, +8];
Alfa1=[-90, -8, -4, -2, 0, +2, +4, +8, +90];

[CX2,CY2,CZ2,Cl2,Cm2,Cn2]= Stage2_Aero_Coeff;                                
Mach2=[ 3,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 20,100];
Beta2=[-3, -1, -0.3, 0, 0.3, 1, 3];
Alfa2=[-20,-17,-14,-12,-10,-8,-6,-5,-4,-3,-2,-1,-0.5,-0.3,-0.1,0, ...
        0.1,0.3,0.5,1,2,3,4,5,6,8,10,12,14,17,20];

% Load FCS Gains .......................................................
load FCS_Gains_Stg1 -ascii
timgn1  = FCS_Gains_Stg1(:, 1);                 % Time (gains)
Vrelgn1 = FCS_Gains_Stg1(:, 3);                 % Gains Veloc Relative
Qbar1   = FCS_Gains_Stg1(:, 4);                 % Gains Dynamic Pressure
Cza1    = FCS_Gains_Stg1(:, 5);                 % Gains Cz_alpha

Km_thet1= FCS_Gains_Stg1(:, 6);                 % Pitch Kq_theta
Km_q1   = FCS_Gains_Stg1(:, 7);                 % Pitch Kq_q (pitch rate)
Km_alfa1= FCS_Gains_Stg1(:, 8);                 % Pitch Kq_alpha
Km_thin1= FCS_Gains_Stg1(:, 9);                 % Pitch Kq_theta_integral
Km_alfin= FCS_Gains_Stg1(:,10);                 % Pitch Kq_theta_integral
Kl_phi1 = FCS_Gains_Stg1(:,11);                 % Roll Kp_phi (roll attitude)
Kl_p1   = FCS_Gains_Stg1(:,12);                 % Roll Kp_p (roll rate)
Kn_psi1 = FCS_Gains_Stg1(:,13);                 % Yaw Kr_psi (yaw attitude)
Kn_r1   = FCS_Gains_Stg1(:,14);                 % Yaw Kr_r (yaw rate)
Kn_beta1= FCS_Gains_Stg1(:,15);                 % Yaw Kr_beta
Kn_psin1= FCS_Gains_Stg1(:,16);                 % Yaw Kr_psi_integral
Kn_betin= FCS_Gains_Stg1(:,17);                 % Yaw Kr_psi_integral

W_nfilt1= FCS_Gains_Stg1(:,20);                 % Slosh Notch Filtr Frequ
K_estim1= FCS_Gains_Stg1(:,21);                 % Alpha Estimator Gain
W_lowp1 = FCS_Gains_Stg1(:,22);                 % Low Pass Filter Frequency

ee=zeros(16); for i=1:17; ee(i,18-i)=1; end
Massgn1 = ee*FCS_Gains_Stg1(:,2);               % Vehicle Mass Transposed
Kml1_1  = ee*FCS_Gains_Stg1(:,18);              % Transpose Kml1_1 Mix Logic
Kml2_1  = ee*FCS_Gains_Stg1(:,19);              % Transpose Kml2_1 Mix Logic

% Load propulsion inputs ...............................................
Isp1=315.0069; Isp2=369.0259;                   % Isp
mdot1=2.5058;  mdot2=2.5058;                    % Flowrates
    
load FCS_Gains_Stg2 -ascii
timgn2  = FCS_Gains_Stg2(:,1);                  % Time (gains)
Vrelgn2 = FCS_Gains_Stg2(:,3);                  % Gains Veloc Relative
Km_thet2= FCS_Gains_Stg2(:,4);                  % Pitch Kq_theta
Km_q2   = FCS_Gains_Stg2(:,5);                  % Pitch Kq_q (pitch rate)
Km_thin2= FCS_Gains_Stg2(:,6);                  % Pitch Kq_theta_integral
Kn_psi2 = FCS_Gains_Stg2(:,7);                  % Yaw Kr_psi (yaw attitude)
Kn_r2   = FCS_Gains_Stg2(:,8);                  % Yaw Kr_r (yaw rate)
Kn_psin2= FCS_Gains_Stg2(:,9);                  % Yaw Kr_psi_integral

ee=zeros(19); for i=1:19; ee(i,20-i)=1; end
Massgn2 = ee*FCS_Gains_Stg2(:,2);               % Vehicle Mass Transposed
Kml2_2  = ee*FCS_Gains_Stg2(:,10);              % Transpose Kml2_1 Mix Logic

% load masses from reference trajectory
fairing_mass = 1047.7*kg2slug;
i_s2_startup = 1744;
s1_init_prop_mass = 3.8002e+03;
s2_init_prop_mass = 801.7606;
total_init_mass = 5.2952e+03;
s2_init_mass = 977.7057;
s1_init_mass = 4.2457e+03;
s2_dry_mass = 175.9451;
s1_dry_mass = 445.4969;
